@@ -2,51 +2,79 @@ use crate::buildstructor::utils::{
     AngleBracketedGenericArgumentsExt, ExprTupleExt, GenericsExt, IdentExt, TypeExt, TypeTupleExt,
 };
 use crate::lower::{FieldType, Ir};
-use proc_macro2::{Ident, TokenStream};
+use proc_macro2::{Ident, Span, TokenStream};
 use quote::{format_ident, quote};
 use syn::punctuated::Punctuated;
-use syn::{Expr, ExprCall, GenericArgument, Generics, Index, Result, Type, WhereClause};
+use syn::{
+    Expr, ExprCall, GenericArgument, GenericParam, Generics, Index, Lifetime, LifetimeDef, Result,
+    Type, WhereClause,
+};
 extern crate inflector;
 use inflector::Inflector;
 
 pub fn codegen(ir: Ir) -> Result<TokenStream> {
     let module_name = &ir.module_name;
-    let target_name = &ir.target_name;
+    let target_name = &ir.impl_name;
 
-    let (impl_generics, ty_generics, where_clause) = &ir.generics.split_for_impl();
+    let (impl_generics, ty_generics, where_clause) = &ir.impl_generics.split_for_impl();
 
     let param_generics = ir.param_generics();
 
-    let builder_generics =
-        Generics::combine(vec![&ir.generics, &ir.method_generics, &param_generics]);
+    let builder_lifetime = ir
+        .receiver
+        .as_ref()
+        .and_then(|r| r.reference.as_ref())
+        .map(|_| Lifetime::new("'__builder", Span::call_site()));
+
+    let builder_lifetime_generics = Generics {
+        params: builder_lifetime
+            .as_ref()
+            .map(|l| {
+                Punctuated::from_iter(vec![GenericParam::Lifetime(LifetimeDef::new(l.clone()))])
+            })
+            .unwrap_or_default(),
+        ..Default::default()
+    };
+
+    let builder_generics = Generics::combine(vec![
+        &builder_lifetime_generics,
+        &ir.impl_generics,
+        &ir.delegate_generics,
+        &param_generics,
+    ]);
     let (builder_impl_generics, _, builder_where_clause) = builder_generics.split_for_impl();
 
-    let builder_generics_tuple = Generics::combine(vec![&ir.generics, &ir.method_generics]);
+    let builder_generics_tuple = Generics::combine(vec![
+        &builder_lifetime_generics,
+        &ir.impl_generics,
+        &ir.delegate_generics,
+    ]);
     let builder_tuple_ty_generics = builder_generics_tuple
         .to_generic_args()
         .insert(0, Type::Tuple(param_generics.to_tuple_type()));
 
     let all_generics = Generics::combine(vec![
+        &builder_lifetime_generics,
         &ir.builder_generics,
-        &ir.generics,
-        &ir.method_generics,
+        &ir.impl_generics,
+        &ir.delegate_generics,
     ]);
     let (_, all_ty_generics, _) = all_generics.split_for_impl();
 
-    let method_generics = &ir.method_generics;
-    let builder_init_generics = Generics::combine(vec![&ir.generics, &ir.method_generics]);
+    let method_generics = &ir.delegate_generics;
+    let builder_init_generics = Generics::combine(vec![&ir.impl_generics, &ir.delegate_generics]);
     let target_generics_raw: Vec<GenericArgument> = builder_init_generics
         .to_generic_args()
         .args
         .into_iter()
         .collect();
 
-    let constructor_method_name = &ir.constructor_method_name;
-    let constructor_args = ir.constructor_args();
-    let constructor_return = &ir.return_type;
-    let builder_method_name = &ir.builder_method_name;
+    let delegate_name = &ir.delegate_name;
+    let delegate_args = ir.delegate_args();
     let builder_name = &ir.builder_name;
-    let builder_methods = builder_methods(&ir, builder_where_clause)?;
+    let builder_return_type = &ir.builder_return_type;
+    let builder_entry = &ir.builder_entry;
+    let builder_exit = &ir.builder_exit;
     let builder_state_type_initial = ir.builder_state_type_initial();
     let builder_state_initial = ir.builder_state_initial();
 
@@ -54,20 +82,53 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
     let await_token = ir.is_async.then(|| quote! {.await});
     let vis = &ir.vis;
     let builder_vis = &ir.builder_vis;
+    let receiver = &ir.receiver;
+    let reference = &ir
+        .receiver
+        .as_ref()
+        .map(|r| r.reference.as_ref().map(|_| quote! { & }));
+    let mutability = ir.receiver.as_ref().map(|r| r.mutability);
+    let builder_receiver = ir.receiver.as_ref().map(|_| quote! { self });
+    let builder_receiver_param = ir
+        .receiver
+        .as_ref()
+        .map(|_| quote! { receiver: #reference #mutability #target_name #ty_generics });
+    let builder_receiver_field_definition = ir
+        .receiver
+        .as_ref()
+        .map(|_| quote! { receiver: #reference #builder_lifetime #mutability #target_name #ty_generics, });
+    let builder_receiver_field = ir.receiver.as_ref().map(|_| quote! { receiver, });
+    let builder_receiver_call = ir
+        .receiver
+        .as_ref()
+        .map(|_| quote! { self.receiver. })
+        .unwrap_or_else(|| quote! {#target_name::});
+    let builder_receiver_move = ir
+        .receiver
+        .as_ref()
+        .map(|_| quote! { receiver: self.receiver, });
+
+    let builder_methods = builder_methods(
+        &ir,
+        builder_where_clause,
+        builder_receiver_move,
+        &builder_lifetime_generics,
+    )?;
 
     Ok(quote! {
         impl #impl_generics #target_name #ty_generics #where_clause {
-            #vis fn #builder_method_name #method_generics() -> #module_name::#builder_name<#builder_state_type_initial, #(#target_generics_raw), *> {
-                #module_name::new()
+            #vis fn #builder_entry #method_generics(#receiver) -> #module_name::#builder_name<#builder_state_type_initial, #(#target_generics_raw), *> {
+                #module_name::new(#builder_receiver)
             }
         }
 
         mod #module_name {
             use super::*;
 
-            pub fn new #builder_init_generics() -> #builder_name<#builder_state_type_initial, #(#target_generics_raw), *>
+            #builder_vis fn new #builder_init_generics(#builder_receiver_param) -> #builder_name<#builder_state_type_initial, #(#target_generics_raw), *>
             {
                 #builder_name {
+                    #builder_receiver_field
                     fields: (#(#builder_state_initial ,) *),
                     _phantom: core::default::Default::default()
                 }
@@ -108,7 +169,8 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
             }
 
 
-            pub struct #builder_name #all_ty_generics {
+            #builder_vis struct #builder_name #all_ty_generics {
+                #builder_receiver_field_definition
                 fields: __P,
                 _phantom: (#(core::marker::PhantomData<#target_generics_raw>,) *)
             }
@@ -116,8 +178,8 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
             #(#builder_methods)*
 
             impl #builder_impl_generics #builder_name #builder_tuple_ty_generics #builder_where_clause {
-                #builder_vis #async_token fn build(self) #constructor_return {
-                    #target_name::#constructor_method_name(#(#constructor_args),*) #await_token
+                #builder_vis #async_token fn #builder_exit(self) #builder_return_type {
+                    #builder_receiver_call #delegate_name(#(#delegate_args),*) #await_token
                 }
             }
         }
@@ -127,8 +189,16 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
 pub fn builder_methods(
     ir: &Ir,
     builder_where_clause: Option<&WhereClause>,
+    builder_receiver_move: Option<TokenStream>,
+    builder_lifetime_generics: &Generics,
 ) -> Result<Vec<TokenStream>> {
-    let builder_generics = Generics::combine(vec![&ir.generics, &ir.method_generics]);
+    let builder_generics = Generics::combine(vec![
+        builder_lifetime_generics,
+        &ir.impl_generics,
+        &ir.delegate_generics,
+    ]);
+    let builder_vis = &ir.builder_vis;
+
     Ok(ir.builder_fields
         .iter()
         .enumerate()
@@ -185,16 +255,18 @@ pub fn builder_methods(
                     }
                     quote! {
                         impl #builder_type_generics #builder_name #before {
-                            pub fn #method_name #into_generics(self, #field_name: #field_collection_type) -> #builder_name #after #builder_where_clause {
+                            #builder_vis fn #method_name #into_generics(self, #field_name: #field_collection_type) -> #builder_name #after #builder_where_clause {
                                 let #field_name = Some(#field_name #into_call);
                                 #builder_name {
+                                    #builder_receiver_move
                                     fields: #new_state,
                                     _phantom: core::default::Default::default()
                                 }
                             }
-                            pub fn #and_method_name #into_generics(self, #field_name: Option<#field_collection_type>) -> #builder_name #after #builder_where_clause {
+                            #builder_vis fn #and_method_name #into_generics(self, #field_name: Option<#field_collection_type>) -> #builder_name #after #builder_where_clause {
                                 let #field_name = #field_name.map(|v|v #into_call);
                                 #builder_name {
+                                    #builder_receiver_move
                                     fields: #new_state,
                                     _phantom: core::default::Default::default()
                                 }
@@ -220,12 +292,12 @@ pub fn builder_methods(
                     quote! {
                         impl #builder_type_generics #builder_name #before {
 
-                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause {
+                            #builder_vis fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause {
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).extend(#field_name.into_iter());
                                 self
                             }
 
-                            pub fn #singular #into_generics(mut self, value: #field_collection_type) -> #builder_name #before #builder_where_clause{
+                            #builder_vis fn #singular #into_generics(mut self, value: #field_collection_type) -> #builder_name #before #builder_where_clause{
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).insert(value #into_call);
                                 self
                             }
@@ -252,12 +324,12 @@ pub fn builder_methods(
                     quote! {
                         impl #builder_type_generics #builder_name #before {
 
-                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause{
+                            #builder_vis fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause{
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).extend(#field_name.into_iter());
                                 self
                             }
 
-                            pub fn #singular #into_generics(mut self, value: #field_collection_type) -> #builder_name #before #builder_where_clause{
+                            #builder_vis fn #singular #into_generics(mut self, value: #field_collection_type) -> #builder_name #before #builder_where_clause{
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).push(value #into_call);
                                 self
                             }
@@ -305,12 +377,12 @@ pub fn builder_methods(
                     quote! {
                         impl #builder_type_generics #builder_name #before {
 
-                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause{
+                            #builder_vis fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause{
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).extend(#field_name.into_iter());
                                 self
                             }
 
-                            pub fn #singular #into_generics_final (mut self, key: #field_key_type, value: #field_value_type) -> #builder_name #before {
+                            #builder_vis fn #singular #into_generics_final (mut self, key: #field_key_type, value: #field_value_type) -> #builder_name #before {
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).insert(key #field_key_into_call, value #field_value_into_call);
                                 self
                             }
@@ -332,9 +404,10 @@ pub fn builder_methods(
                     }
                     quote! {
                         impl #builder_type_generics #builder_name #before {
-                            pub fn #method_name #into_generics(self, #field_name: #ty) -> #builder_name #after {
+                            #builder_vis fn #method_name #into_generics(self, #field_name: #ty) -> #builder_name #after {
                                 let #field_name = #field_name #into_call;
                                 #builder_name {
+                                    #builder_receiver_move
                                     fields: #new_state,
                                     _phantom: core::default::Default::default()
                                 }
@@ -395,9 +468,9 @@ mod tests {
 
     macro_rules! assert_codegen {
         ($input:expr) => {
-            let models = analyze($input).expect("Analysis failed");
+            let models = analyze(false, &$input).expect("Analysis failed");
             for model in models {
-                let ir = lower(model).expect("Ir failed");
+                let ir = lower(model.expect("Analysis failed")).expect("Ir failed");
                 if let Ok(codegen) = codegen(ir) {
                     if let Ok(new_ast) = syn::parse2(codegen.clone()) {
                         let output = prettyplease::unparse(&new_ast);
@@ -486,4 +559,9 @@ mod tests {
     fn collection_generics_test2() {
         assert_codegen!(collections_generics_test_case2());
     }
+
+    #[test]
+    fn self_receiver() {
+        assert_codegen!(self_receiver_test_case());
+    }
 }
