@@ -11,8 +11,9 @@ mod tests {
 
     pub fn single_field_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new(simple: usize) -> Foo {
                     Self { simple }
                 }
@@ -22,8 +23,9 @@ mod tests {
 
     pub fn pub_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 pub fn new(simple: usize) -> Foo {
                     Self { simple }
                 }
@@ -33,8 +35,9 @@ mod tests {
 
     pub fn multi_field_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new(simple: usize, simple2: usize) -> Foo {
                     Self { simple, simple2 }
                 }
@@ -44,8 +47,9 @@ mod tests {
 
     pub fn fallible_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new(simple: usize) -> Result<Foo, String> {
                     Ok(Self { simple })
                 }
@@ -55,8 +59,9 @@ mod tests {
 
     pub fn async_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 async fn new(simple: usize) -> Foo {
                     Foo { simple }
                 }
@@ -66,8 +71,9 @@ mod tests {
 
     pub fn generic_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl<T> Foo<T> {
+                #[builder]
                 fn new(simple: T) -> Foo<T> {
                     Self { simple }
                 }
@@ -77,8 +83,9 @@ mod tests {
 
     pub fn into_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new<T: Into<String>>(simple: T) -> Foo {
                     Foo {
                         simple: simple.into(),
@@ -90,8 +97,9 @@ mod tests {
 
     pub fn into_where_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new<T>(simple: T) -> Foo
                 where
                     T: Into<String>,
@@ -106,8 +114,9 @@ mod tests {
 
     pub fn option_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new(option: Option<usize>) -> Foo {
                     Foo { option }
                 }
@@ -117,8 +126,9 @@ mod tests {
 
     pub fn collections_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new(
                     simple: usize,
                     set: HashSet<String>,
@@ -142,8 +152,9 @@ mod tests {
 
     pub fn collections_generics_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new<K: Into<String> + Eq + Hash, V: Into<String>>(param: HashMap<K, V>) -> Foo {
                     Self {
                         param: param
@@ -158,8 +169,9 @@ mod tests {
 
     pub fn collections_option_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new(param: HashMap<Option<String>, Option<String>>) -> Foo {
                     Self {
                         param: param
@@ -174,8 +186,9 @@ mod tests {
 
     pub fn returns_self_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Foo {
+                #[builder]
                 fn new(simple: usize) -> Self {
                     Self { simple }
                 }
@@ -185,8 +198,9 @@ mod tests {
 
     pub fn multiple_generics_test_case() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl<T> Request<T> {
+                #[builder]
                 pub fn fake_new<K, V>(
                     headers: Vec<(K, V)>,
                     uri: Option<http::Uri>,
@@ -213,8 +227,9 @@ mod tests {
 
     pub fn collections_generics_test_case2() -> Ast {
         parse_quote!(
-            #[builder]
+            #[buildstructor]
             impl Collections {
+                #[builder]
                 fn new<K: Into<String> + Eq + Hash, V: Into<String>>(
                     map: HashMap<K, V>,
                     set: HashSet<K>,
@@ -227,4 +242,27 @@ mod tests {
             }
         )
     }
+
+    pub fn self_receiver_test_case() -> Ast {
+        parse_quote!(
+            #[buildstructor]
+            impl Client {
+                #[builder(entry = "message", exit = "send")]
+                fn call_with_no_return(self, simple: String) {}
+
+                #[builder(entry = "message_ref", exit = "send")]
+                fn call_with_no_return_ref(&self, simple: String) {}
+
+                #[builder(entry = "query", exit = "call")]
+                fn call_with_return(self, simple: String) -> bool {
+                    true
+                }
+
+                #[builder(entry = "query_ref", exit = "call")]
+                fn call_with_return_ref(&self, simple: String) -> bool {
+                    true
+                }
+            }
+        )
+    }
 }
