@@ -1,11 +1,12 @@
-use buildstructor::builder;
+use buildstructor::buildstructor;
 
 pub struct Foo {
     simple: usize,
 }
 
-#[builder]
+#[buildstructor]
 impl Foo {
+    #[builder]
     async fn new(simple: usize) -> Foo {
         Foo { simple }
     }
