@@ -1,16 +1,19 @@
-use buildstructor::builder;
+use buildstructor::buildstructor;
 pub struct Foo {
     simple: usize,
 }
 
-#[builder]
+#[buildstructor]
 impl Foo {
+    #[builder]
     fn new(simple: usize) -> Result<Foo, String> {
         Ok(Self { simple })
     }
+    #[builder]
     fn try_new(simple: usize) -> Result<Foo, String> {
         Ok(Self { simple })
     }
+    #[builder]
     fn maybe_new(simple: usize) -> Result<Foo, String> {
         Ok(Self { simple })
     }
