@@ -1,10 +1,11 @@
-use buildstructor::builder;
+use buildstructor::buildstructor;
 pub struct Foo {
     simple: usize,
 }
 
-#[builder]
+#[buildstructor]
 impl Foo {
+    #[builder]
     fn new(simple: usize) -> Self {
         Self { simple }
     }
